import { Injectable } from '@angular/core';
import { CanActivate, Router, ActivatedRouteSnapshot, CanActivateChild, RouterStateSnapshot, UrlTree } from '@angular/router';

import { ToasterWrapperService } from '../_services/toaster-wrapper.service';
import { AuthenticationService } from '../_services/authentication.service';
import { Observable, of } from 'rxjs';
import { PwaStatusService } from '../_services/pwa-status.service';
import { map, tap } from 'rxjs/operators';

@Injectable()
export class RequiresOnlineGuard implements CanActivateChild {

  constructor(private pwaStatusServ: PwaStatusService, private router: Router, private toastService: ToasterWrapperService) {}

  canActivateChild(childRoute: ActivatedRouteSnapshot, state: RouterStateSnapshot) {

    const requiresOnlineKey = 'requiresOnline';
    const requiresOnline = childRoute.data[requiresOnlineKey];

    if (!requiresOnline){
      return of(true);
    }

    return this.pwaStatusServ.status$()
      .pipe(
        map(s => s.isOnline),
        tap(s => {
          if (!s){
            this.toastService.toastError('Per accedere alla pagina selezionata è necessario essere online');
          }
        })
      );
  }
}
