import { Injectable } from '@angular/core';
import { HttpEvent, HttpHandler, HttpInterceptor, HttpRequest, HTTP_INTERCEPTORS } from '@angular/common/http';
import { Observable } from 'rxjs';
import { finalize } from 'rxjs/operators';
import { SpinnerWrapperService } from '../_services/spinner-wrapper.service';
import { ExclusionHeaders } from './exclusion-headers';

@Injectable()
export class SpinnerInterceptor implements HttpInterceptor {
    constructor(private spinnerServ: SpinnerWrapperService) { }
    intercept(req: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
        
        if (req.headers.has(ExclusionHeaders.SPINNER_INTERCEPTOR)){
            return next.handle(req);
        }
        
        console.log('start ' + req.urlWithParams);
        this.spinnerServ.show();
        
        return next.handle(req).pipe(
            finalize(() => {console.log('finalize ' + req.urlWithParams);  this.spinnerServ.hide(); })
        );
    }
}

export const SpinnerInterceptorProvider = {
    provide: HTTP_INTERCEPTORS,
    useClass: SpinnerInterceptor,
    multi: true
};
