import { SafeUrl } from '@angular/platform-browser';
import { IHasTranslations } from '../_interfaces/has-translations';

export class Utente {
    // da tenere allineato con enum LivelloDipendente in backend
    public static readonly LIVELLO_DIPENDENTE_DEFAULT = 0;

    Id: number;
    Username: string;
    LivelloAccesso: number;
    LivelloAccessoString: string;
    Nome: string;
    Cognome: string;
    Matricola: string;
    Mail: string;
    IdSAP: string;
    BadgeNFC: string;
    LivelloDipendente: number;
    LivelloDipendenteString: string;
    Abilitato: boolean;
    IsUtenteLoggato: boolean;
    IdentificativoUtente: string;
    PermessiString: string;
    Permessi: Elemento[];
}
export class ProdottoDB implements IHasTranslations {
    Codice: string;
    Descrizione: string;
    Abilitato: BooleanState;

    Pics: ElementoPicDB[];
    Traduzioni: TraduzioneDB[];
}
export class GruppoDB implements IHasTranslations {
    CodiceParent: string;
    Codice: string;
    Descrizione: string;
    Abilitato: BooleanState;

    Pics: ElementoPicDB[];
    Traduzioni: TraduzioneDB[];
}
export class ParticolareDB implements IHasTranslations {
    CodiceParent: string;
    Codice: string;
    Descrizione: string;
    Abilitato: BooleanState;

    Traduzioni: TraduzioneDB[];
}
export class ComponenteDB implements IHasTranslations {
    CodiceParent: string;
    Codice: string;
    Descrizione: string;
    Abilitato: BooleanState;

    Traduzioni: TraduzioneDB[];
}
export class AnomaliaDB implements IHasTranslations {
    Codice: string;
    Descrizione: string;
    Abilitato: BooleanState;
    Audit: BooleanState;

    Traduzioni: TraduzioneDB[];
}
export class PosizioneAnomaliaDB implements IHasTranslations {
    Codice: string;
    Descrizione: string;
    Abilitato: BooleanState;

    Traduzioni: TraduzioneDB[];
}
export class PesoDifettoDB {
    Id: number;
    CodiceOld: string;
    CodiceNew: string;
    TipoDifetto: TipoDifetto;
    Abilitato: BooleanState;
}
export class ResponsabilitaDB {
    Id: number;
    Nome: string;
    Descrizione: string;
    GammaId: number;
    GammaNome: string;
    Abilitato: BooleanState;
}
export class GammaDB {
    Id: number;
    Nome: string;
    Descrizione: string;
    Abilitato: BooleanState;
}
export class GammaSpecificaDB {
    GammaId: number;
    GammaNome: string;
    Id: number;
    Nome: string;
    Descrizione: string;
    Abilitato: BooleanState;
}

export class ChecklistDB {
    Id: number;
    Nome: string;
    Descrizione: string;
    Abilitato: BooleanState;
    Tipologia: TipoDifetto;
    TipoAcquisizionePdi: TipoAcquisizione;
    NomeCommessa: string;
}

export class FullChecklistDB extends ChecklistDB {
    Elementi: ElementoChecklistDB[];
}

export enum TipoAcquisizione {
    Normale = 0,
    Commessa = 1
}

export class ElementoChecklistDB {
    Id: number;
    ComponenteCodice: string;
    AnomaliaCodice: string;
    PosizioneAnomaliaCodice: string;
    PesoDifettoId: number;
    Ordinamento: number;
    Descrizione: string;
    Abilitato: BooleanState;

    ChecklistId: number;
    Base64Pic: string;

    Traduzioni: TraduzioneDB[];
}

export enum BooleanState {
    False = 0,
    True = 1
}

export class TraduzioneDB {
    Nome: string;
    Code: string;
    Descrizione: string;
}
export class ElementoPicDB {
    Gamma: string;
    IdGamma: number;
    Base64Pic: string;
    ThumbnailBase64Pic: string;
}

export class VeicoloDB {
    // calcolato in base a VAN e Categoria
    GUID?: string;

    VAN: string;

    // per gestione audit in base a categoria
    Categoria: string;

    GAN: string;
    PIC: string;
    VIN: string;
    MiniDesc: string;
    LatoGuida: VeicoloLatoGuida;
    Scudato: VeicoloScudato;
    Passo: number;
    GammaId?: number;
    GammaSpecificaId?: number;
    DataProduzione?: Date;
}

export class FullVeicoloDB extends VeicoloDB {
    Sessioni: FullSessioneDB[];
}

export enum VeicoloLatoGuida {
    Invalido = 0,
    SX = 1,
    DX = 2
}

export enum VeicoloScudato {
    Invalido = 0,
    No = 1,
    Si = 2
}
export class SessioneDB {
    // generato tramite uuidjs
    GUID?: string;

    Tipo: TipoSessione;
    TipoAudit: TipoAudit;
    Stato: StatoSessione;

    DataInizioAcquisizione?: Date;
    MatricolaUtenteInizioAcquisizione: string;

    DataFineAcquisizione?: Date;
    MatricolaUtenteFineAcquisizione: string;

    DataInizioReworking?: Date;
    MatricolaUtenteInizioReworking: string;

    DataFineReworking?: Date;
    MatricolaUtenteFineReworking: string;

    VeicoloGUID: string;
    ChecklistId: number;

    // todo difetti extra pdi
}

export class FullSessioneDB extends SessioneDB {
    Difetti: StoricoDifettoDB[];
}

export enum TipoSessione {
    Pdi = 0,
    Audit = 1,
    Mrl = 2
}

export enum TipoAudit {
    MQA = 0, // audit completo
    CPA = 1 // audit 6 ore
}

export enum StatoSessione {
    InizioAcquisizione = 0,
    FineAcquisizione = 1,
    InizioReworking = 2,
    FineReworking = 3
}

export class StoricoDifettoDB {
    // generato tramite uuidjs
    GUID?: string;

    Codice: string;
    Stato: StatoStoricoDifetto;
    Tipo: TipoStoricoDifetto;

    // per gerarchia prodotto -> gruppo -> particolare -> componente
    ProdottoCodice: string;
    GruppoCodice: string;
    ParticolareCodice?: string;
    ComponenteCodice?: string;

    // sarà la descrizione di una delle anomalie fra quelle associate
    // al componente (al momento della registrazione del difetto)
    DifettoCodice: string;

    // direttamente da anag. posizioni difetto
    PosizioneDifettoCodice: string;

    PesoDifettoId: number;

    TipoDifetto?: TipoDifetto;

    ResponsabilitaIds: number[];
    OriginiIds: number[];

    XCoord: number;
    YCoord: number;

    NoteCreazione: string;
    NoteRisoluzione: string;

    DataCreazione?: Date;
    DataRisoluzione?: Date;

    MatricolaUtenteCreazione: string;
    MatricolaUtenteRisoluzione: string;

    SessioneGUID: string;
    OrdinamentoPdi?: number;
    DifettoNonRiscontrabilePdi?: boolean;
    DifettoExtraPdi?: boolean;

    Base64Pic: string;
}

export class DetailedStoricoDifetto extends StoricoDifettoDB {
    ProdottoDescrizione: string;
    GruppoDescrizione: string;
    ParticolareDescrizione: string;
    ComponenteDescrizione: string;
    DifettoDescrizione: string;
    PosizioneDifettoDescrizione: string;
    PesoDifettoOldCodice: string;
    PesoDifettoNewCodice: string;

    ResponsabilitaString: string;
    OriginiString: string;
}

// dichiarato dall'operatore in fase di creazione del difetto stesso
export enum TipoDifetto {
    Estetico = 0,
    Funzionale = 1
}

export enum TipoStoricoDifetto {
    Pdi = 0,
    Audit = 1,
    Mrl = 2
}

export enum StatoStoricoDifetto {
    DaAnalizzare = 0, // tutti i difetti PDI nascono in questo stato
    Presente = 1, // stato iniziale difetti AUDIT; stato difetti pdi se l'operatore indica "NON OK"/"NON RISCONTRABILE" in acquisizione PDI
    Risolto = 2, // stato finale possibile difetti AUDIT/PDI se il difetto viene risolto
    NonRisolvibile = 3, // stato finale possibile difetti AUDIT/PDI se il difetto NON può essere risolto (non ne vale la pena)
    NonPresente = 4 // stato per i difetti PDI se l'operatore indica "OK"
}

export class Elemento {
    public static readonly VALORE_NO = 0;
    public static readonly VALORE_SI = 1;

    Id: number;
    Nome: string;
}

export class VarianteElemento {
    Id: string;
    Nome: string;
}

export class ElementoGroup extends Elemento {
    Gruppo: string;
}

export class IntIdModel {
    Id: number;
}

export class StringIdModel {
    Id: string;
}

export class Base64StringModel {
    Base64String: string;
}

export class GetBase64FullDocument {
    RelativePath: string;
    FileName: string;
}

export class AppSettings {
    CurrentEnvironment: string;
    CurrentPlant: string;
    AutoLogoutMinuti: number;
}