import { StaticProvider, Type } from '@angular/core';
import { GridsterItem } from 'angular-gridster2';
import { gridTypes, displayGrids } from 'angular-gridster2/lib/gridsterConfig.interface';

export class DashboardConfig {
  public Title: string;
  public Subtitle: string;

  public GridType?: gridTypes;
  public DisplayGrid?: displayGrids;
  public MinCols?: number;
  public MaxCols?: number;
  public MinRows?: number;
  public MaxRows?: number;
  public FixedRowHeight?: number;
  public FixedColWidth?: number;

  public AllItems: Array<CustomGridsterItem>;
  public DefaultItems: Array<CustomGridsterItem>;
  public Items: Array<CustomGridsterItem>;
  public AvailableItems: Array<CustomGridsterItem>;
}

export class CustomGridsterItem implements GridsterItem {
  x: number;
  y: number;
  rows: number;
  cols: number;
  // già presenti ma rese obbligatorie
  minItemRows: number;
  minItemColumns: number;
  // aggiunte
  id: number;
  type: string;
  component?: Type<any>;
  title: string;
  icon?: string;
  headerComponent?: Type<any>;
  buttonsComponents?: Type<any>[];
  // Per ogni pannello, verrà creata un'istanza di ogni servizio definito
  // nell'array: tali istanze saranno quindi accessibili ai componenti che costituiscono
  // il pannello: headerComponent, buttonsComponent e component(il principale).
  // Il punto fondamentale è che due pannelli differenti non condividono tali servizi
  sharedServices?: any[];
  // dati addizionali necessari per alcuni pannelli (ad es. adaptive reports)
  customData?: object;
}

export interface IDashboardConfigService {
  getAvailableItems(): CustomGridsterItem[];
  getConfig(): DashboardConfig;
  updateItems(items: CustomGridsterItem[]): void;
}

export abstract class DashboardConfigServiceBase implements IDashboardConfigService {
  abstract getAvailableItems(): CustomGridsterItem[];
  abstract getConfig(): DashboardConfig;
  abstract updateItems(items: CustomGridsterItem[]): void;
}

