import { BehaviorSubject, Observable } from 'rxjs';
import { distinctUntilChanged, map } from 'rxjs/operators';
import { UtilsService } from '../utils.service';

export class StateService<T> {
  private state$: BehaviorSubject<T>;

  protected get state(): T {
    return this.state$.getValue();
  }

  constructor(initialState: T, protected utilsServ: UtilsService) {
    this.initialize(initialState);
  }

  public initialize(initialState: T){
    this.state$ = new BehaviorSubject<T>(initialState);
  }

  public select<K>(mapFn: (state: T) => K, noClone: boolean = false): Observable<K> {
    return this.state$.asObservable().pipe(
      map((state: T) => mapFn(state)),
      distinctUntilChanged(),
      map(selected => noClone ? selected : this.utilsServ.clone(selected))
    );
  }

  protected setState(newState: Partial<T>) {
    this.state$.next({
      ...this.state,
      ...newState,
    });
  }
}