import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { of } from 'rxjs';
import { debounceTime, distinctUntilChanged, map, flatMap, takeLast } from 'rxjs/operators';
import { environment } from 'src/environments/environment';
import { SearchElementiEntita } from '../_models/search-elementi-entita';
import { VarianteElemento } from '../_models/common';

@Injectable()
export class AnomaliaSearchService
{
    private URLSearchAnomalie = environment.apiUrl + '/anomalie/search';
    private numeroMinimoCaratteriTestoRicerca = 3;

    constructor(private httpClient: HttpClient) {}

    public searchAnomalie(testoRicerca: string)
    {
        if (!testoRicerca || testoRicerca.length < this.numeroMinimoCaratteriTestoRicerca)
        {
            return of(null);
        }
        else
        {
            return of(testoRicerca).pipe(
                debounceTime(400),
                takeLast(1),
                distinctUntilChanged(),
                map(m => this._getSearchAnomalie(testoRicerca)),
                flatMap(m => this._searchAnomalie(m))
            );
        }
    }

    private _getSearchAnomalie(testoRicerca: string)
    {
        const searchAnomalie = new SearchElementiEntita();
        searchAnomalie.TestoRicerca = testoRicerca;

        return searchAnomalie;
    }

    private _searchAnomalie(searchAnomalie: SearchElementiEntita)
    {
        if (!searchAnomalie.TestoRicerca)
            return of(null);

        else
            return this.httpClient.post<VarianteElemento[]>(this.URLSearchAnomalie, searchAnomalie);
    }
}