import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { of } from 'rxjs';
import { debounceTime, distinctUntilChanged, map, flatMap, takeLast } from 'rxjs/operators';
import { environment } from 'src/environments/environment';
import { SearchElementiEntita } from '../_models/search-elementi-entita';
import { VarianteElemento } from '../_models/common';

@Injectable()
export class ComponenteSearchService
{
    private URLSearchComponenti = environment.apiUrl + '/componenti/search';
    private numeroMinimoCaratteriTestoRicerca = 3;

    constructor(private httpClient: HttpClient) {}

    public searchComponenti(testoRicerca: string, codiceEntitaAssociata: string)
    {
        if (!testoRicerca || testoRicerca.length < this.numeroMinimoCaratteriTestoRicerca)
        {
            return of(null);
        }
        else
        {
            return of(testoRicerca).pipe(
                debounceTime(400),
                takeLast(1),
                distinctUntilChanged(),
                map(m => this._getSearchComponenti(testoRicerca, codiceEntitaAssociata)),
                flatMap(m => this._searchComponenti(m))
            );
        }
    }

    private _getSearchComponenti(testoRicerca: string, codiceEntitaAssociata: string)
    {
        const searchComponenti = new SearchElementiEntita();
        searchComponenti.TestoRicerca = testoRicerca;
        searchComponenti.CodiceEntitaAssociata = codiceEntitaAssociata;

        return searchComponenti;
    }

    private _searchComponenti(searchComponenti: SearchElementiEntita)
    {
        if (!searchComponenti.TestoRicerca)
            return of(null);

        else
            return this.httpClient.post<VarianteElemento[]>(this.URLSearchComponenti, searchComponenti);
    }
}