import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { of } from 'rxjs';
import { debounceTime, distinctUntilChanged, map, flatMap, takeLast } from 'rxjs/operators';
import { environment } from 'src/environments/environment';
import { SearchElementiEntita } from '../_models/search-elementi-entita';
import { VarianteElemento } from '../_models/common';

@Injectable()
export class GruppoSearchService
{
    private URLSearchGruppi = environment.apiUrl + '/gruppi/search';
    private numeroMinimoCaratteriTestoRicerca = 3;

    constructor(private httpClient: HttpClient) {}

    public searchGruppi(testoRicerca: string, codiceEntitaAssociata: string)
    {
        if (!testoRicerca || testoRicerca.length < this.numeroMinimoCaratteriTestoRicerca)
        {
            return of(null);
        }
        else
        {
            return of(testoRicerca).pipe(
                debounceTime(400),
                takeLast(1),
                distinctUntilChanged(),
                map(m => this._getSearchGruppi(testoRicerca, codiceEntitaAssociata)),
                flatMap(m => this._searchGruppi(m))
            );
        }
    }

    private _getSearchGruppi(testoRicerca: string, codiceEntitaAssociata: string)
    {
        const searchGruppi = new SearchElementiEntita();
        searchGruppi.TestoRicerca = testoRicerca;
        searchGruppi.CodiceEntitaAssociata = codiceEntitaAssociata;

        return searchGruppi;
    }

    private _searchGruppi(searchGruppi: SearchElementiEntita)
    {
        if (!searchGruppi.TestoRicerca)
            return of(null);

        else
            return this.httpClient.post<VarianteElemento[]>(this.URLSearchGruppi, searchGruppi);
    }
}