import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { of } from 'rxjs';
import { debounceTime, distinctUntilChanged, map, flatMap, takeLast } from 'rxjs/operators';
import { environment } from 'src/environments/environment';
import { SearchElementiEntita } from '../_models/search-elementi-entita';
import { VarianteElemento } from '../_models/common';

@Injectable()
export class ParticolareSearchService
{
    private URLSearchParticolari = environment.apiUrl + '/particolari/search';
    private numeroMinimoCaratteriTestoRicerca = 3;

    constructor(private httpClient: HttpClient) {}

    public searchParticolari(testoRicerca: string, codiceEntitaAssociata: string)
    {
        if (!testoRicerca || testoRicerca.length < this.numeroMinimoCaratteriTestoRicerca)
        {
            return of(null);
        }
        else
        {
            return of(testoRicerca).pipe(
                debounceTime(400),
                takeLast(1),
                distinctUntilChanged(),
                map(m => this._getSearchParticolari(testoRicerca, codiceEntitaAssociata)),
                flatMap(m => this._searchParticolari(m))
            );
        }
    }

    private _getSearchParticolari(testoRicerca: string, codiceEntitaAssociata: string)
    {
        const searchParticolari = new SearchElementiEntita();
        searchParticolari.TestoRicerca = testoRicerca;
        searchParticolari.CodiceEntitaAssociata = codiceEntitaAssociata;

        return searchParticolari;
    }

    private _searchParticolari(searchParticolari: SearchElementiEntita)
    {
        if (!searchParticolari.TestoRicerca)
            return of(null);

        else
            return this.httpClient.post<VarianteElemento[]>(this.URLSearchParticolari, searchParticolari);
    }
}