import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { of } from 'rxjs';
import { debounceTime, distinctUntilChanged, map, flatMap, takeLast } from 'rxjs/operators';
import { environment } from 'src/environments/environment';
import { SearchElementiEntita } from '../_models/search-elementi-entita';
import { Elemento } from '../_models/common';

@Injectable()
export class PesoDifettoSearchService
{
    private URLSearchPesiDifetti = environment.apiUrl + '/pesidifetti/search';
    private numeroMinimoCaratteriTestoRicerca = 1;

    constructor(private httpClient: HttpClient) {}

    public searchPesiDifetti(testoRicerca: string)
    {
        if (!testoRicerca || testoRicerca.length < this.numeroMinimoCaratteriTestoRicerca)
        {
            return of(null);
        }
        else
        {
            return of(testoRicerca).pipe(
                debounceTime(400),
                takeLast(1),
                distinctUntilChanged(),
                map(m => this._getSearchPesiDifetti(testoRicerca)),
                flatMap(m => this._searchPesiDifetti(m))
            );
        }
    }

    private _getSearchPesiDifetti(testoRicerca: string)
    {
        const searchPesiDifetti = new SearchElementiEntita();
        searchPesiDifetti.TestoRicerca = testoRicerca;

        return searchPesiDifetti;
    }

    private _searchPesiDifetti(searchPesiDifetti: SearchElementiEntita)
    {
        if (!searchPesiDifetti.TestoRicerca)
            return of(null);

        else
            return this.httpClient.post<Elemento[]>(this.URLSearchPesiDifetti, searchPesiDifetti);
    }
}