import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { of } from 'rxjs';
import { debounceTime, distinctUntilChanged, map, flatMap, takeLast } from 'rxjs/operators';
import { environment } from 'src/environments/environment';
import { SearchElementiEntita } from '../_models/search-elementi-entita';
import { VarianteElemento } from '../_models/common';

@Injectable()
export class PosizioneAnomaliaSearchService
{
    private URLSearchPosAnom = environment.apiUrl + '/posizionianomalie/search';
    private numeroMinimoCaratteriTestoRicerca = 2;

    constructor(private httpClient: HttpClient) {}

    public searchPosizioniAnomalie(testoRicerca: string)
    {
        if (!testoRicerca || testoRicerca.length < this.numeroMinimoCaratteriTestoRicerca)
        {
            return of(null);
        }
        else
        {
            return of(testoRicerca).pipe(
                debounceTime(400),
                takeLast(1),
                distinctUntilChanged(),
                map(m => this._getSearchPosAnom(testoRicerca)),
                flatMap(m => this._searchPosAnom(m))
            );
        }
    }

    private _getSearchPosAnom(testoRicerca: string)
    {
        const searchPosAnom = new SearchElementiEntita();
        searchPosAnom.TestoRicerca = testoRicerca;

        return searchPosAnom;
    }

    private _searchPosAnom(searchPosAnom: SearchElementiEntita)
    {
        if (!searchPosAnom.TestoRicerca)
            return of(null);

        else
            return this.httpClient.post<VarianteElemento[]>(this.URLSearchPosAnom, searchPosAnom);
    }
}