import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { of } from 'rxjs';
import { debounceTime, distinctUntilChanged, map, flatMap, takeLast } from 'rxjs/operators';
import { environment } from 'src/environments/environment';
import { SearchElementiEntita } from '../_models/search-elementi-entita';
import { VarianteElemento } from '../_models/common';

@Injectable()
export class ProdottoSearchService
{
    private URLSearchProdotti = environment.apiUrl + '/prodotti/search';
    private numeroMinimoCaratteriTestoRicerca = 2;

    constructor(private httpClient: HttpClient) {}

    public searchProdotti(testoRicerca: string)
    {
        if (!testoRicerca || testoRicerca.length < this.numeroMinimoCaratteriTestoRicerca)
        {
            return of(null);
        }
        else
        {
            return of(testoRicerca).pipe(
                debounceTime(400),
                takeLast(1),
                distinctUntilChanged(),
                map(m => this._getSearchProdotti(testoRicerca)),
                flatMap(m => this._searchProdotti(m))
            );
        }
    }

    private _getSearchProdotti(testoRicerca: string)
    {
        const searchProdotti = new SearchElementiEntita();
        searchProdotti.TestoRicerca = testoRicerca;

        return searchProdotti;
    }

    private _searchProdotti(searchProdotti: SearchElementiEntita)
    {
        if (!searchProdotti.TestoRicerca)
            return of(null);

        else
            return this.httpClient.post<VarianteElemento[]>(this.URLSearchProdotti, searchProdotti);
    }
}