import { Injectable } from '@angular/core';
import { BehaviorSubject } from 'rxjs';
import { Settings } from '../_models/settings';
import { environment } from 'src/environments/environment';
import { HttpClient } from '@angular/common/http';
import { AppSettings } from '../_models/common';

@Injectable()
export class SettingsService {

    private app: Settings;
    private appSettingsURL = environment.apiUrl + '/appsettings/get-app-settings';

    private $hiddenLayout = new BehaviorSubject<boolean>(false);

    constructor(private http: HttpClient) {

        const cs = JSON.parse(localStorage.getItem('currentSettings'));

        if (cs){
            this.app = cs as Settings;
        } else {
            this.app = {
                name: '',
                company: '',
                version: '',
                currentEnvironment: '',
                currentPlant: '',
                autoLogoutMinuti: 30,
                year: ((new Date()).getFullYear()),
                language: 'it',
                layout:  {
                    isFixed: true,
                    isCollapsed: false,
                    isBoxed: false,
                    isRTL: false,
                    horizontal: false,
                    isFloat: false,
                    asideHover: false,
                    asideScrollbar: false,
                    isCollapsedText: false,
                    useFullLayout: false,
                    hiddenFooter: false,
                    offsidebarOpen: false,
                    asideToggled: false,
                    viewAnimation: 'ng-fadeInUp'
                }
            };
        }

        this.app.name = 'SMART audit';
        this.app.company = 'TRIM Informatica S.r.l.';
        this.app.version = '1.0.0';
        this.app.currentEnvironment = '-';
        this.app.currentPlant = '-';

/*         this.http.get<AppSettings>(this.appSettingsURL)
            .subscribe(m => {
                this.app.currentEnvironment = m.CurrentEnvironment;
                this.app.currentPlant = m.CurrentPlant;
                this.app.autoLogoutMinuti = m.AutoLogoutMinuti;
            }); */
    }

    isLayoutHidden(){
        return this.$hiddenLayout.asObservable();
    }

    setLayoutHidden(isHidden: boolean){
        this.$hiddenLayout.next(isHidden);
    }

    getAppSetting(name) {
        return name ? this.app[name] : this.app;
    }
    getLayoutSetting(name) {
        return name ? this.app.layout[name] : this.app.layout;
    }

    setAppSetting(name, value) {
        if (typeof this.app[name] !== 'undefined') {
            this.app[name] = value;
            localStorage.setItem('currentSettings',  JSON.stringify(this.app));
        }
    }
    setLayoutSetting(name, value) {
        if (typeof this.app.layout[name] !== 'undefined') {
            const toReturn = this.app.layout[name] = value;
            localStorage.setItem('currentSettings',  JSON.stringify(this.app));
            return toReturn;
        }
    }

    toggleLayoutSetting(name) {
        return this.setLayoutSetting(name, !this.getLayoutSetting(name));
    }
}
