import { BehaviorSubject, Observable } from 'rxjs';
import { Injectable } from '@angular/core';

@Injectable()
export class SpinnerWrapperService {

    private showSpinnerSubject: BehaviorSubject<boolean>;
    public showSpinner: Observable<boolean>;

    // a fronte di richieste al backend da parte di più
    // resolver può verificarsi il seguente ordine di eventi:
    // 1) start prima richiesta
    // 2) start seconda richiesta
    // 3) finalize prima richiesta
    // 4) finalize seconda richiesta
    // => questo porterebbe al fatto che lo spinner verrebbe nascosto prima
    // che la seconda richiesta abbia terminato (e cioè al punto 3).
    // Per ovviare al problema, il contatore "pendingRequests" mi permette
    // di sapere quante richieste pendenti ci sono: in questo modo, l'hide viene davvero
    // eseguito solo all'ultimo finalize, e cioè quando non ho più richieste pendenti
    private pendingRequests: number;

    constructor() {
        this.showSpinnerSubject = new BehaviorSubject<boolean>(false);
        this.showSpinner = this.showSpinnerSubject.asObservable();
        this.pendingRequests = 0;
    }

    show() {
        if (this.pendingRequests <= 0){
            this.showSpinnerSubject.next(true);
            console.log('SHOW SPINNER');
        }

        this.pendingRequests++;
        console.log('pendingRequests++ -> current:' + this.pendingRequests);
    }

    hide() {
        this.pendingRequests--;
        console.log('pendingRequests-- -> current: ' + this.pendingRequests);

        if (this.pendingRequests <= 0){
            this.showSpinnerSubject.next(false);
            console.log('HIDE SPINNER');
        }
    }
}