import { Injectable } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { setCurrencyCode, setCulture, Ajax, L10n, loadCldr } from '@syncfusion/ej2-base';
import { LanguageData } from '../_models/language-data';
import { SettingsService } from './settings.service';
import am4lang_it_IT from '@amcharts/amcharts4/lang/it_IT';
import am4lang_en_US from '@amcharts/amcharts4/lang/en_US';
import * as am4core from '@amcharts/amcharts4/core';

@Injectable()
export class TranslatorService {

    private defaultLanguage = 'it';

    private currentLanguage: LanguageData;

    private availablelangs: LanguageData[] = [
        { code: 'it', currencyCode: 'EUR', ej2Locale: 'it', amchartsLocale: am4lang_it_IT, cldrData: 'it', name: 'Italiano', dateTimeFormat: 'dd/MM/yyyy HH:mm', dateFormat: 'dd/MM/yyyy' },
        { code: 'en-US', currencyCode: 'USD',  ej2Locale: 'en-US', amchartsLocale: am4lang_en_US, cldrData: 'en', name: 'English', dateTimeFormat: 'MM/dd/yyyy HH:mm', dateFormat: 'MM/dd/yyyy' }
    ];

    constructor(private translate: TranslateService, private settings: SettingsService) {
        this.translate.addLangs(this.availablelangs.map(s => s.code));

        this.currentLanguage = this.getLanguageData(this.settings.getAppSetting('language'));

        translate.setDefaultLang(this.defaultLanguage);
        this.useLanguage(this.currentLanguage.code);
    }

    public get(key){
        return this.translate.instant(key);
    }

    public getLanguage(){
        return this.currentLanguage;
    }

    private getLanguageData(code: string): LanguageData {
        return this.availablelangs.filter(s => s.code === code)[0];
    }

    useLanguage(lang: string) {

        this.currentLanguage = this.getLanguageData(lang);
        this.settings.setAppSetting('language', lang);

        this.translate.use(this.currentLanguage.code);

        // EJ2 -> testi
        setCulture(this.currentLanguage.ej2Locale);

        const ajax = new Ajax('/assets/i18n/ej2/' +  this.currentLanguage.ej2Locale + '.json', 'GET', true);
        ajax.onSuccess = (value) => {
            const toLoad = JSON.parse(value);

            if (toLoad.it) {
                // fix stringhe di traduzione errate
                toLoad.it.dropdowns.overflowCountTemplate = '${count} di più ..';
                toLoad.it.dropdowns.totalCountTemplate = '${count} selezionato';

                toLoad.it['multi-select'].overflowCountTemplate = '${count} di più ..';
                toLoad.it['multi-select'].totalCountTemplate = '${count} selezionato';
            }

            L10n.load(toLoad);
        };
        ajax.send();

        // EJ2 -> formati. ATTENZIONE!! usando il valore di languageData.cldrData
        // all'interno dei vari require per evitare di duplicare i require a fronte
        // di lingue differenti ho poi un problema in fase di ng build --prod perchè
        // webpack le esamina e include tutti i file possibili che corrispondono al pattern
        // RISULTATO: -> 1) vengono incluse anche tutte le lingue che non mi servono
        // 2) il file main.. .js compilato ha una dimensione eccessiva (oltre 20 MB!)
        loadCldr(
            require('cldr-data/main/it/numbers.json'),
            require('cldr-data/main/it/ca-gregorian.json'),
            require('cldr-data/main/en/numbers.json'),
            require('cldr-data/main/en/ca-gregorian.json'),
            require('cldr-data/supplemental/numberingSystems.json'),
            require('cldr-data/main/it/timeZoneNames.json'),
            require('cldr-data/main/en/timeZoneNames.json'),
            require('cldr-data/supplemental/weekData.json'), // To load the culture based first day of week
            require('cldr-data/main/it/currencies.json'),
            require('cldr-data/main/en/currencies.json')
          );

        // EJ2 -> simbolo valuta
        setCurrencyCode(this.currentLanguage.currencyCode);

        // set locale amcharts
        am4core.options.defaultLocale = this.currentLanguage.amchartsLocale;
    }

    getAvailableLanguages() {
        return this.availablelangs;
    }

    getGridDateTimeFormat() {
        return { type: 'date', format: this.getLanguage().dateTimeFormat };
    }

    getGridDateFormat() {
        return { type: 'date', format: this.getLanguage().dateFormat };
    }

    getDatePickerDateTimeFormat() {
        return this.getLanguage().dateTimeFormat;
    }

    getDatePickerDateFormat() {
        return this.getLanguage().dateFormat;
    }
}
