import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { environment } from 'src/environments/environment';
import { SearchUtenti } from '../_models/search-utenti';
import { Elemento } from '../_models/common';
import { of } from 'rxjs';
import { debounceTime, distinctUntilChanged, map, takeLast, switchMap } from 'rxjs/operators';

@Injectable()
export class UserSearchService {

constructor(private http: HttpClient) { }

  private utentiSearchURL = environment.apiUrl + '/users/search';
  private minQueryLength = 3;

  search(text, userLevels, includeDisabled, ids = null){
    if ((!text || text.length < this.minQueryLength) && !ids){
      return of(null);
    }

    return of(text)
    .pipe(
      debounceTime(400),
      takeLast(1),
      distinctUntilChanged(),
      map(text => {
      const query = new SearchUtenti();

      query.Ids = ids;
      query.QueryIdentificativo = text;
      query.LivelliDipendenti = userLevels;
      query.IncludiDisabilitati = includeDisabled;
      return query;
    }), switchMap(query => this.http.post<Elemento[]>(this.utentiSearchURL, query)));
  }

}
