import { NgModule, Optional, SkipSelf } from '@angular/core';

import { SettingsService } from './_services/settings.service';
import { TranslatorService } from './_services/translator.service';
import { MenuService } from './_services/menu.service';
import { throwIfAlreadyLoaded } from './module-import-guard';
import { FilterService, PageService, SortService, GroupService, ToolbarService, DetailRowService, ExcelExportService, ResizeService } from '@syncfusion/ej2-angular-grids';
import { ErrorInterceptorProvider } from './_interceptors/error.interceptor';
import { AuthenticationService } from './_services/authentication.service';
import { ToasterWrapperService } from './_services/toaster-wrapper.service';
import { RoleAuthGuard } from './_guards/role-auth.guard';
import { SpinnerWrapperService } from './_services/spinner-wrapper.service';
import { SpinnerInterceptorProvider } from './_interceptors/spinner.interceptor';
import { TokenInterceptorProvider } from './_interceptors/token.interceptor';
import { ColorsService } from './_services/colors.service';
import { UserblockService } from './_services/userblock.service';
import { SpinnerWrapperComponent } from './spinner-wrapper/spinner-wrapper.component';
import { CommonModule } from '@angular/common';
import { UtilsService } from './_services/utils.service';
import { ToastrModule } from 'ngx-toastr';
import { ProdottoSearchService } from './_services/prodotto-search.service';
import { GruppoSearchService } from './_services/gruppo-search.service';
import { ParticolareSearchService } from './_services/particolare-search.service';
import { ComponenteSearchService } from './_services/componente-search.service';
import { AnomaliaSearchService } from './_services/anomalia-search.service';
import { PosizioneAnomaliaSearchService } from './_services/posizione-anomalia-search.service';
import { PesoDifettoSearchService } from './_services/peso-difetto-search.service';
import { UserSearchService } from './_services/user-search.service';
import { NavigationPaneService, DetailsViewService, ToolbarService as FileManagerToolbarService }
 from '@syncfusion/ej2-angular-filemanager';
import { SitewideSearchService } from './_services/sitewide-search.service';
import { ChatSignalRService } from './_services/chat-signalr.service';
import { NgxIndexedDBModule } from 'ngx-indexed-db';
import { dbConfig, DbWrapperService } from './_services/db-wrapper.service';
import { PwaStatusService } from './_services/pwa-status.service';
import { RequiresOnlineGuard } from './_guards/requires-online.guard';

@NgModule({
    imports: [
        CommonModule,
        ToastrModule.forRoot({
            positionClass: 'toast-top-right',
            preventDuplicates: true
        }),
        NgxIndexedDBModule.forRoot(dbConfig)
    ],
    providers: [
        // servizi SINGLETON default
        SettingsService,
        TranslatorService,
        MenuService,
        ColorsService,
        UserblockService,
        // servizi SINGLETON aggiunti (_services)
        AuthenticationService,
        ProdottoSearchService,
        GruppoSearchService,
        ParticolareSearchService,
        ComponenteSearchService,
        AnomaliaSearchService,
        PosizioneAnomaliaSearchService,
        PesoDifettoSearchService,
        UserSearchService,
        ToasterWrapperService,
        SpinnerWrapperService,
        SitewideSearchService,
        ChatSignalRService,
        UtilsService,
        DbWrapperService,
        PwaStatusService,
        // GUARDS
        RoleAuthGuard,
        RequiresOnlineGuard,
        // EJ2 Syncfusion (in questo modo i servizi sono forniti come singleton
        // anche se SharedModule viene utilizzato da moduli che funzionano in lazy-loading)
        PageService,
        SortService,
        FilterService,
        GroupService,
        ToolbarService,
        // ResizeService, (per ora non utilizzato)
        DetailRowService,

        NavigationPaneService,
        FileManagerToolbarService,
        DetailsViewService,
        ExcelExportService,

        // HTTP interceptors
        TokenInterceptorProvider,
        ErrorInterceptorProvider,
        SpinnerInterceptorProvider
    ],
    declarations: [
        SpinnerWrapperComponent
    ],
    exports: [
        SpinnerWrapperComponent
    ]
})
export class CoreModule {
    constructor( @Optional() @SkipSelf() parentModule: CoreModule) {
        throwIfAlreadyLoaded(parentModule, 'CoreModule');
    }
}
