import { Component, OnDestroy, OnInit, ViewEncapsulation } from '@angular/core';
import { Subscription } from 'rxjs';

declare var Stimulsoft: any;

@Component({
  selector: 'app-print-difetti-sessione',
  templateUrl: './print-difetti-sessione.component.html',
  styleUrls: ['./print-difetti-sessione.component.scss'],
  encapsulation: ViewEncapsulation.None
})
export class PrintDifettiSessioneComponent implements OnInit, OnDestroy {

  options: any = new Stimulsoft.Viewer.StiViewerOptions();
  viewer: any = new Stimulsoft.Viewer.StiViewer(this.options, 'StiViewer', false);
  

  private subscriptions = new Subscription();

  constructor() { }

  ngOnInit() {
    const testataDataSet = new Stimulsoft.System.Data.DataSet('TestataDeliberaDto');
    testataDataSet.readJson({ TestataDeliberaDto: {
      VAN: 'VAN',
      GAN: 'GAN',
      Mini_desc: 'mini_desc',
      DataProduzione: new Date(),
      DataInizioAcquisizione: new Date(),
      TotalePesoNewDifettiEstetici: 1,
      TotalePesoNewDifettiFunzionali: 2,
      TotalePesoOldDifettiEstetici: 3,
      TotalePesoOldDifettiFunzionali: 4,
    } });

    const difettiDataSet = new Stimulsoft.System.Data.DataSet('DifettoDeliberaDto');
    difettiDataSet.readJson({ DifettoDeliberaDto: [{
        Codice: 'code 1',
        Descrizione: 'description 1',
        ListaResponsabilita: 'a, b',
        PesoCodiceOld: '1',
        PesoCodiceNew: '2',
        Tipo: 'type 1',
        NoteCreazione: 'notes 1',
        // these same pic bytes generate a small picture if the report is generated on the backend with C# code: on the front-end nothing is displayed instead..
        ImmagineDifetto: [137, 80, 78, 71, 13, 10, 26, 10, 0, 0, 0, 13, 73, 72, 68, 82, 0, 0, 0, 3, 0, 0, 0, 3, 8, 6, 0, 0, 0, 86, 40, 181, 191, 0, 0, 0, 1, 115, 82, 71, 66, 0, 174, 206, 28, 233, 0, 0, 0, 4, 103, 65, 77, 65, 0, 0, 177, 143, 11, 252, 97, 5, 0, 0, 0, 9, 112, 72, 89, 115, 0, 0, 14, 194, 0, 0, 14, 194, 1, 21, 40, 74, 128, 0, 0, 0, 27, 73, 68, 65, 84, 24, 87, 99, 124, 43, 163, 242, 159, 1, 10, 24, 255, 215, 171, 35, 56, 64, 12, 229, 48, 48, 0, 0, 150, 205, 5, 211, 149, 46, 77, 162, 0, 0, 0, 0, 73, 69, 78, 68, 174, 66, 96, 130]
      },
      {
        Codice: 'code 2',
        Descrizione: 'description 2',
        ListaResponsabilita: 'b, c',
        PesoCodiceOld: '2',
        PesoCodiceNew: '3',
        Tipo: 'type 2',
        NoteCreazione: 'notes 2',
        // these same pic bytes generate a small picture if the report is generated on the backend with C# code: on the front-end nothing is displayed instead..
        ImmagineDifetto: [137, 80, 78, 71, 13, 10, 26, 10, 0, 0, 0, 13, 73, 72, 68, 82, 0, 0, 0, 3, 0, 0, 0, 3, 8, 6, 0, 0, 0, 86, 40, 181, 191, 0, 0, 0, 1, 115, 82, 71, 66, 0, 174, 206, 28, 233, 0, 0, 0, 4, 103, 65, 77, 65, 0, 0, 177, 143, 11, 252, 97, 5, 0, 0, 0, 9, 112, 72, 89, 115, 0, 0, 14, 194, 0, 0, 14, 194, 1, 21, 40, 74, 128, 0, 0, 0, 27, 73, 68, 65, 84, 24, 87, 99, 124, 43, 163, 242, 159, 1, 10, 24, 255, 215, 171, 35, 56, 64, 12, 229, 48, 48, 0, 0, 150, 205, 5, 211, 149, 46, 77, 162, 0, 0, 0, 0, 73, 69, 78, 68, 174, 66, 96, 130]
    }] });

    const report = new Stimulsoft.Report.StiReport();
    report.loadFile('assets/reports/TestReport.mrt');

    report.regData(testataDataSet.dataSetName, '', testataDataSet);
    report.regData(difettiDataSet.dataSetName, '', difettiDataSet);

    this.options.toolbar.viewMode = Stimulsoft.Viewer.StiWebViewMode.Continuous;

    this.viewer.report = report;
    this.viewer.renderHtml('viewer');
  }

  ngOnDestroy(){
    this.subscriptions.unsubscribe();
  }
}

export class TestataDeliberaDto {
  VAN: string;
  GAN: string;
  Mini_desc: string;
  DataProduzione: Date;
  DataInizioAcquisizioneAudit: Date;
  TotalePesoNewDifettiEstetici: number;
  TotalePesoNewDifettiFunzionali: number;
  TotalePesoOldDifettiEstetici: number;
  TotalePesoOldDifettiFunzionali: number;
}

export class DifettoDeliberaDto {
  Codice: string;
  Descrizione: string;
  ListaResponsabilita: string;
  PesoCodiceOld: string;
  PesoCodiceNew: string;
  Tipo: string;
  NoteCreazione: string;
  ImmagineDifetto: number[];
}
