import { isPlatformBrowser } from '@angular/common';
import { Component, Inject, Input, NgZone, OnDestroy, OnInit, PLATFORM_ID } from '@angular/core';

// amCharts imports
import * as am4core from '@amcharts/amcharts4/core';
import * as am4charts from '@amcharts/amcharts4/charts';
import am4themes_animated from '@amcharts/amcharts4/themes/animated';

@Component({
  selector: 'app-am4charts-wrapper',
  templateUrl: './am4charts-wrapper.component.html',
  styleUrls: ['./am4charts-wrapper.component.scss']
})
export class Am4chartsWrapperComponent implements OnInit, OnDestroy {

  @Input()
  chartId: string = null;

  private chart: am4core.Container;

  constructor(@Inject(PLATFORM_ID) private platformId, private zone: NgZone) {}

  ngOnInit(){}

  // Run the function only in the browser
  browserOnly(f: () => void) {
    if (isPlatformBrowser(this.platformId)) {
      this.zone.runOutsideAngular(() => {
        f();
      });
    }
  }

  initChart(chartProvider:(() => am4charts.Chart)){
    this._disposeChart();

    // Chart code goes in here
    this.browserOnly(() => {
      am4core.useTheme(am4themes_animated);

      if (!chartProvider){
        console.log('chartProvider not set => no chart painted');
        return;
      }

      const chart = chartProvider();

      if (!chart){
        console.log('chartProvider returned null chart => chart not painted');
        return;
      }

      console.log('chart initialized: \"' + this.chartId + '\"');
      this.chart = chart;
    });
  }

  ngOnDestroy() {
    this._disposeChart();
  }

  private _disposeChart(){
    // Clean up chart when the component is removed
    this.browserOnly(() => {
      if (this.chart) {
        this.chart.dispose();
      }
    });
  }
}
