import { Component, OnInit, EventEmitter, Output, Input, OnChanges } from '@angular/core';
import { Router } from '@angular/router';
import { CardData, CardGridOptions } from 'src/app/core/_models/card-grid';
import {CdkDragDrop, moveItemInArray} from '@angular/cdk/drag-drop';
import { number } from '@amcharts/amcharts4/core';

@Component({
  selector: 'app-card-grid',
  templateUrl: './card-grid.component.html',
  styleUrls: ['./card-grid.component.scss']
})
export class CardGridComponent implements OnInit {

  private _selectedItem: CardData;

  @Output()
  moveItem = new EventEmitter<{ item: CardData, newIndex: number}>();

  @Output()
  editItem = new EventEmitter<CardData>();

  @Output()
  deleteItem = new EventEmitter<CardData>();

  @Output()
  dragDrop = new EventEmitter<{ previousIndex: number, currentIndex: number}>();

  @Output()
  selectedItemChange = new EventEmitter<CardData>();

  @Input()
  editMode = false;

  @Input()
  get selectedItem() {
    return this._selectedItem;
  }
  set selectedItem(val) {
    if (val !== this._selectedItem){
      this._selectedItem = val;
      this.selectedItemChange.emit(this._selectedItem);
    }
  }

  @Input()
  dataSource: Array<CardData>;

  private _options: CardGridOptions = { 
    CardHeight: '200px',
    CardMinWidth: '300px',
    CardTextSize: '1.875rem', 
    CardSubtextSize:'1.125rem',
    CardBorderRadius: '0.5rem',
    CardHorizontalSpacing: '0px',
    CardVerticalSpacing: '0px',
    CardPictureMaxHeight: '120px',
    CardPictureMinHeight: '120px',
    CardPictureTopMargin: '15px'
  };

  @Input()
  get options(){
    return this._options;
  }
  set options(val){
    if (!val){
      return;
    }

    // partendo dalle impostazioni iniziali, aggiorno le proprietà
    // che hanno dei valori differenti nell'oggetto passato (val)
    this._options = { ...this._options, ...val };
  }

  constructor(private router: Router) { }

  onMoveItemLeft(item: CardData){
    const newIndex = this.dataSource.indexOf(item) - 1;

    this.moveItem.emit({ item, newIndex});
  }

  onMoveItemRight(item: CardData){
    const newIndex = this.dataSource.indexOf(item) + 1;
    this.moveItem.emit({ item, newIndex });
  }

  onEditItem(item: CardData){
    this.editItem.emit(item);
  }

  onDeleteItem(item: CardData){
    this.deleteItem.emit(item);
  }

  ngOnInit() {
  }

/*   getBackgroundColorWithGradient(item: CardData){
    if (!item.Color){
      return undefined;
    }

    const lighterColor = '#ffffff';//pSBC(item.Color, 0.15);

    return 'linear-gradient(to right, ' + item.Color + ',' + lighterColor + ')';
  }

  private _lightenDarkenColor(col: string, lightenCoefficient: number) {
    let toParse = col.replace('#', '').toUpperCase();

    const amt = lightenCoefficient * 100;

    const parsed = parseInt(toParse, 16);
    return '#' + (((parsed & 0x0000FF) + amt) | ((((parsed >> 8) & 0x00FF) + amt) << 8) | (((parsed >> 16) + amt) << 16)).toString(16);
  } */

  getLeftRightMargin(){
    const horizontalSpacing = parseInt(this._options.CardHorizontalSpacing, 10);

    const halfSpacing = horizontalSpacing / 2;

    const suffix = this._options.CardHorizontalSpacing.replace(/\d+/g,'');

    return halfSpacing.toString() + suffix;
  }

  selectionClass(item) {
    if (item === this._selectedItem) {
      return 'selected-card';
    }
    return '';
  }

  onSelectItem(item) {
    // no selection in edit mode
    if (this.editMode){
      return;
    }

    if (this._selectedItem === item) {
      this._selectedItem = null;
      this.selectedItemChange.emit(null);
    } else {
      this._selectedItem = item;
      this.selectedItemChange.emit(item);

      if (this._selectedItem.LinkUrl){
        setTimeout(() => this.router.navigate([this._selectedItem.LinkUrl]), 500);
      }
    }
  }

  drop(event: CdkDragDrop<string[]>) {
    this.dragDrop.emit({currentIndex: event.currentIndex, previousIndex: event.previousIndex });
  }
}

