import { Component, OnInit, Input, Output, EventEmitter, ComponentRef, Injector, ViewEncapsulation } from '@angular/core';
import { IResizeListener } from 'src/app/core/_interfaces/resize-listener';
import { CustomGridsterItem } from 'src/app/core/_models/dashboard';

@Component({
  selector: 'app-dashboard-item',
  templateUrl: './dashboard-item.component.html',
  styleUrls: ['./dashboard-item.component.scss'],
  encapsulation: ViewEncapsulation.None
})
export class DashboardItemComponent implements OnInit {

  @Input()
  item: CustomGridsterItem;

  maximized = false;

  innerComponent:IResizeListener;

  @Output()
  itemRemoveRequested = new EventEmitter<CustomGridsterItem>();

  @Output()
  itemMaxSizeRequested = new EventEmitter<CustomGridsterItem>();

  @Output()
  itemNormalSizeRequested = new EventEmitter<CustomGridsterItem>();

  constructor() {}

  ngOnInit() {}

  getCustomData(){
    if (this.item.customData){
      return { customData: this.item.customData }
    } else {
      return null;
    }
  }

  componentCreated(compRef: ComponentRef<any>) {
    this.innerComponent = this.instanceOfIResizeListener(compRef.instance)
      ? compRef.instance as IResizeListener
      : null;
  }

  instanceOfIResizeListener(object: any): object is IResizeListener {
      return 'onResize' in object;
  }

  onClickRemove(evt) {
    evt.preventDefault();
    evt.stopPropagation();
    this.itemRemoveRequested.emit(this.item);
  }

  onClickMaxSize(evt){
    evt.preventDefault();
    evt.stopPropagation();

    this.maximized = true;

    this.itemMaxSizeRequested.emit(this.item);
    this.toggleInnerMaximized();
  }

  onClickNormalSize(evt){
    evt.preventDefault();
    evt.stopPropagation();

    this.maximized = false;

    this.itemNormalSizeRequested.emit(this.item);
    this.toggleInnerMaximized();
  }

  toggleInnerMaximized(){
    if (this.innerComponent){
      this.innerComponent.onResize(this.maximized);
    }
  }
}
