import { Component, EventEmitter, Input, OnChanges, OnInit, Output } from '@angular/core';
import { MenuItemModel } from '@syncfusion/ej2-angular-navigations';
import { CustomGridsterItem } from 'src/app/core/_models/dashboard';

@Component({
  selector: 'app-dashboard-panels-menu',
  templateUrl: './dashboard-panels-menu.component.html',
  styleUrls: ['./dashboard-panels-menu.component.scss']
})
export class DashboardPanelsMenuComponent implements OnInit, OnChanges {

  @Input()
  availablePanels: Array<CustomGridsterItem>;

  @Output()
  resetPanels = new EventEmitter<any>();

  @Output()
  addPanel = new EventEmitter<CustomGridsterItem>();

  private _resetPanelsMenuItem = {
      id: 'reset-panels',
      text: 'Resetta pannelli',
      iconCss: 'e-icons e-refresh',
  };

  dashboardMenu: MenuItemModel[];

  constructor() { }

  ngOnInit() {
  }

  ngOnChanges(){
    console.log('CHANGES IN PANELS MENU');

    const availablePanelsMenuItem = this.generateAvailablePanelsMenuItem();

    if (availablePanelsMenuItem.items.length <= 0){
      this.dashboardMenu = [this._resetPanelsMenuItem];
    } else{
      this.dashboardMenu = [this._resetPanelsMenuItem, availablePanelsMenuItem];
    }
  }

  generateAvailablePanelsMenuItem(){
    return {
      id: 'add-panels',
      text: 'Pannelli disponibili',
      iconCss: 'fa fa-plus-square',
      items: this.availablePanels
      .map(m => { return { id: m.type, text: m.title };})
    };
  }

  getAvailablePanel(type){
    const matchingPanels = this.availablePanels.filter(m => m.type === type);

    if (matchingPanels.length <= 0){
      return null;
    }

    return matchingPanels[0];
  }

  onDashboardMenuItemSelect(args){
    switch(args.item.id){
      case 'reset-panels':
        this.resetPanels.emit();
        break;
      default:
        const availablePanel = this.getAvailablePanel(args.item.id);
        if (availablePanel){
          this.addPanel.emit(availablePanel);
        }
        break;
    }
  }
}
