import { Component, OnInit, Input, ElementRef, ViewChild, Output, EventEmitter, ChangeDetectionStrategy, OnChanges } from '@angular/core';
import { DomSanitizer } from '@angular/platform-browser';
import { DetailedStoricoDifetto, StatoStoricoDifetto, TipoDifetto } from 'src/app/core/_models/common';

@Component({
    selector: 'app-grid-difetti-sessione',
    templateUrl: './grid-difetti-sessione.component.html',
    styleUrls: ['./grid-difetti-sessione.component.scss'],
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class GridDifettiSessioneComponent implements OnInit {
    @Input()
    difetti: DetailedStoricoDifetto[];

    @Output()
    goToGruppoDifetto = new EventEmitter<string>();

    @Input()
    shown = false;

    @Output()
    toggleShown = new EventEmitter<any>();

    pageSettings = { pageSize: 5 };

    constructor(private domSanitizer: DomSanitizer) {}

    ngOnInit() {}

    onToggleShown(){
        this.toggleShown.emit();
    }

    onGoToGruppoDifetto(difettoGUID: string){
        this.goToGruppoDifetto.emit(difettoGUID);
    }

    getBadgeStatoDifettoSessione(statoDifettoSessione: StatoStoricoDifetto){
        switch (statoDifettoSessione)
        {
            case StatoStoricoDifetto.DaAnalizzare:
                return 'badge badge-difetto-da-analizzare';

            case StatoStoricoDifetto.NonPresente:
                return 'badge badge-difetto-non-presente';

            case StatoStoricoDifetto.Presente:
                return 'badge badge-difetto-presente';

            case StatoStoricoDifetto.Risolto:
                return 'badge badge-difetto-risolto';

            case StatoStoricoDifetto.NonRisolvibile:
                return 'badge badge-difetto-non-risolvibile';

            default:
                return 'badge badge-difetto-non-riscontrabile';
        }
    }

    getTextStatoDifettoSessione(statoDifettoSessione: StatoStoricoDifetto){
        switch (statoDifettoSessione)
        {
            case StatoStoricoDifetto.DaAnalizzare:
                return 'Da analizzare';

            case StatoStoricoDifetto.NonPresente:
                return 'Non presente';

            case StatoStoricoDifetto.Presente:
                return 'Presente';

            case StatoStoricoDifetto.Risolto:
                return 'Risolto';

            case StatoStoricoDifetto.NonRisolvibile:
                return 'Non risolvibile';

            default:
                return 'Non riscontrabile';
        }
    }

    getBadgeTipoDifettoSessione(tipoDifettoSessione: TipoDifetto){
        switch (tipoDifettoSessione)
        {
            case TipoDifetto.Estetico:
                return 'badge badge-difetto-estetico';

            case TipoDifetto.Funzionale:
                return 'badge badge-difetto-funzionale';
        }
    }

    getTextTipoDifettoSessione(tipoDifettoSessione: TipoDifetto){
        switch (tipoDifettoSessione)
        {
            case TipoDifetto.Estetico:
                return 'Estetico';

            case TipoDifetto.Funzionale:
                return 'Funzionale';
        }
    }
}