import { Component, OnInit, ViewChild, ElementRef, EventEmitter, Output, Input, AfterViewInit } from '@angular/core';

enum NFCDataStatus {
  None = 0,
  OK = 1,
  KO = 2
}

export class NFCData {
  isValid: boolean;
  code: string;
}

@Component({
  selector: 'app-nfc-reader',
  templateUrl: './nfc-reader.component.html',
  styleUrls: ['./nfc-reader.component.scss']
})
export class NfcReaderComponent implements OnInit, AfterViewInit {

  @ViewChild('hiddenNFCField', {static: false})
  hiddenNFCField: ElementRef;

  @Output()
  onReadingCompleted = new EventEmitter<NFCData>();

  @Input()
  isActive = false;

  isReading = false;
  nfcCode = '';

  constructor() { }

  ngOnInit() { }

  ngAfterViewInit() {
    // timeout necessario per evitare
    // eccezione dovuta a logica di funzionamento
    // di angular (non ho approfondito troppo..)
    setTimeout(() => {
      if (this.isActive){
        this.onActivateNFC();
      }
    }, 0);
  }

  getReadingStatus() {
    if (this.isReading) {
      return 'Lettura NFC attiva';
    }
    return 'Attiva lettura NFC';
  }

  onActivateNFC() {
    this.nfcCode = null;
    this.hiddenNFCField.nativeElement.focus();
    this.isReading = true;
  }

  onLostFocus(){
    this.isReading = false;
  }

  getBadgeClass(){
    const dataStatus = this._getDataStatus();

    switch(dataStatus)
    {
      case NFCDataStatus.None:
        return 'no-reading';
      case NFCDataStatus.OK:
        return 'reading-ok';
      case NFCDataStatus.KO:
        return 'reading-ko';
    }
  }

  getDataStatusText(){
    const dataStatus = this._getDataStatus();

    switch(dataStatus)
    {
      case NFCDataStatus.None:
        return 'Nessun codice acquisito';
      case NFCDataStatus.OK:
        return 'Codice acquisito correttamente';
      case NFCDataStatus.KO:
        return 'Il codice acquisito non è valido';
    }
  }

  onKeyUp(evt) {
    // quando leggo il carattere 'Return'..
    if (evt.keyCode === 13) {
      // ..rimuovo manualmente il focus dal campo di input nascosto..
      this.hiddenNFCField.nativeElement.blur();
      // ..e scateno l'evento che segnala l'acquisizione completata,
      // passando come argomento un oggetto che indica il codice letto,
      // e se il codice letto è formalmente valido, 
      // in base alla funzione "_isValidNFC"
      this.onReadingCompleted.emit({ 
        isValid: this._getDataStatus() === NFCDataStatus.OK,
        code: this.nfcCode
      });
    }
  }

  hasData(){
    return this._getDataStatus() !== NFCDataStatus.None;
  }

  clearData(){
    this.nfcCode = null;
    this.onReadingCompleted.emit({ 
      isValid: this._getDataStatus() === NFCDataStatus.OK,
      code: this.nfcCode
    });
  }

  private _getDataStatus(): NFCDataStatus {
    if (!this.nfcCode || this.isReading) {
      return NFCDataStatus.None;
    } else if (this._isValidNFC(this.nfcCode)) {
      return NFCDataStatus.OK;
    } else {
      return NFCDataStatus.KO;
    }
  }

  private _isValidNFC(code: string){
    if (code && code.length > 3){
      return true;
    }

    return false;
  }

// TODO: scrivere una funzione che verifichi la
// correttezza formale del codice NFC inserito
/*   private _isValidNFC(code: string) {
    const codeParts = code.split('-');

    if (codeParts.length !== 4) {
      return false;
    }

    if (codeParts.filter(part => part.length !== 2).length > 0) {
      return false;
    }

    const alphanumericCheck = new RegExp(/^[a-z0-9]+$/i);

    if (codeParts.filter(part => !alphanumericCheck.test(part)).length > 0) {
      return false;
    }

    return true;
  } */
}

