import { Component, OnDestroy, OnInit } from '@angular/core';
import { Subscription } from 'rxjs';
import { PwaStatus, PwaStatusService } from 'src/app/core/_services/pwa-status.service';

@Component({
  selector: 'app-pwa-status',
  templateUrl: './pwa-status.component.html',
  styleUrls: ['./pwa-status.component.scss']
})
export class PwaStatusComponent implements OnInit, OnDestroy {

  private subscriptions = new Subscription();

  status: PwaStatus;
  expanded = false;

  constructor(private pwaStatusServ: PwaStatusService) { }

  ngOnInit() {
    this.subscriptions.add(
      this.pwaStatusServ.status$()
      .subscribe(m => {
        this.status = m;

        if (!this.canExpand()){
          this.expanded = false;
        }
      })
    );
  }

  ngOnDestroy(){
    this.subscriptions.unsubscribe();
  }

  getBadgeClasses(){
    return {
      'badge': true,
      'badge-green': this.status.isOnline,
      'badge-red': !this.status.isOnline,
      'is-clickable': this.canExpand()
    }
  }

  getBadgeOnlineStatusText(){
    return (this.status.isOnline)
      ? 'online'
      : 'offline';
  }

  getPingText(){
    if (!this.status.ping){
      return '...ms';
    }

    return this.status.ping + 'ms';
  }

  canExpand(){
    return !!this.status && this.status.isOnline;
  }

  onToggleExpanded(){
    if (!this.canExpand()){
      return;
    }

    this.expanded = !this.expanded;
  }
}
