import { Component, OnInit, Input, EventEmitter, Output } from '@angular/core';
import { Elemento } from 'src/app/core/_models/common';

export enum RangeType {
  Today = 0,
  Last7Days = 1,
  LastMonth = 2,
  Last3Months = 3,
  Last6Months = 4,
  LastYear = 5,
  Last3Years = 6,
  Last5Years = 7,
  Custom = 8
}

@Component({
  selector: 'app-smart-rangepicker',
  templateUrl: './smart-rangepicker.component.html',
  styleUrls: ['./smart-rangepicker.component.scss']
})
export class SmartRangepickerComponent implements OnInit {

  @Input()
  from?:Date;
  @Output()
  fromChange = new EventEmitter<Date>();

  @Input()
  to?: Date;
  @Output()
  toChange = new EventEmitter<Date>();

  selectedRangeType: RangeType;

  datasourceRangeTypes = [
    { Id: RangeType.Today, Nome: 'Oggi'},
    { Id: RangeType.Last7Days, Nome: 'Ultimi 7 giorni'},
    { Id: RangeType.LastMonth, Nome: 'Ultimo mese'},
    { Id: RangeType.Last3Months, Nome: 'Ultimi 3 mesi'},
    { Id: RangeType.Last6Months, Nome: 'Ultimi 6 mesi'},
    { Id: RangeType.LastYear, Nome: 'Ultimo anno'},
    { Id: RangeType.Last3Years, Nome: 'Ultimi 3 anni'},
    { Id: RangeType.Last5Years, Nome: 'Ultimi 5 anni'},
    { Id: RangeType.Custom, Nome: 'Personalizzata'}
  ];

  ddlFields = { text: 'Nome', value: 'Id' };

  constructor() { }

  ngOnInit() {
    this.from = this.from ? new Date(this.from) : null;
    this.to = this.to ? new Date(this.to) : null;

    this.selectedRangeType = this.getCurrentRange();
  }

  getCurrentRange(){
    if (this.from && !this.to){
      if (this.from.getTime() === this._getDateDaysAgo(0).getTime()){
        return RangeType.Today;
      } else if (this.from.getTime() === this._getDateDaysAgo(7).getTime()){
        return RangeType.Last7Days;
      } else if (this.from.getTime() === this._getDateMonthsAgo(1).getTime()){
        return RangeType.LastMonth;
      } else if (this.from.getTime() === this._getDateMonthsAgo(3).getTime()){
        return RangeType.Last3Months;
      } else if (this.from.getTime() === this._getDateMonthsAgo(6).getTime()){
        return RangeType.Last6Months;
      } else if (this.from.getTime() === this._getDateMonthsAgo(12).getTime()){
        return RangeType.LastYear;
      } else if (this.from.getTime() === this._getDateDaysAgo(12 * 3).getTime()){
        return RangeType.Last3Years;
      } else if (this.from.getTime() === this._getDateDaysAgo(12 * 5).getTime()){
        return RangeType.Last5Years;
      }
    }

    return RangeType.Custom;
  }

  isCustom(){
    return this.selectedRangeType === RangeType.Custom;
  }

  onChangeRangeType(evt) {
    switch (evt.itemData.Id)
    {
      case RangeType.Today:
        this.from = this._getDateDaysAgo(0);
        this.to = null;
        break;
      case RangeType.Last7Days:
        this.from = this._getDateDaysAgo(7);
        this.to = null;
        break;
      case RangeType.LastMonth:
        this.from = this._getDateMonthsAgo(1);
        this.to = null;
        break;
      case RangeType.Last3Months:
        this.from = this._getDateMonthsAgo(3);
        this.to = null;
        break;
      case RangeType.Last6Months:
        this.from = this._getDateMonthsAgo(6);
        this.to = null;
        break;
      case RangeType.LastYear:
        this.from = this._getDateMonthsAgo(12);
        this.to = null;
        break;
      case RangeType.Last3Years:
        this.from = this._getDateMonthsAgo(12 * 3);
        this.to = null;
        break;
      case RangeType.Last5Years:
        this.from = this._getDateMonthsAgo(12 * 5);
        this.to = null;
        break;
      case RangeType.Custom:
        this.from = null;
        this.to = null;
        break;
    }

    this._emitOutputEvents();
  }

  onChangeCustomRange(evt) {
    if (!evt.startDate || !evt.endDate){
      return;
    }

    this.from = evt.startDate;
    this.to = this._getDateEndDay(evt.endDate);

    this._emitOutputEvents();
  }

  private _emitOutputEvents() {
    this.fromChange.emit(this.from);
    this.toChange.emit(this.to);
  }

  private _getDateEndDay(date: Date) {
    const toReturn = new Date(date);
    toReturn.setHours(23, 59, 59, 999);
    return toReturn;
  }

  private _getDateMonthsAgo(monthsAgo: number){
    const toReturn = this._addMonths(new Date(), -monthsAgo);

    // mi interessa solo il giorno
    return new Date(toReturn.toDateString());
  }

  // funzione presa da questa risposta su stackoverflow:
  // https://stackoverflow.com/questions/7937233/how-do-i-calculate-the-date-in-javascript-three-months-prior-to-today
  private _addMonths(date, months) {
    const d = date.getDate();
    date.setMonth(date.getMonth() + +months);
    if (date.getDate() !== d) {
      date.setDate(0);
    }
    return date;
  }

  private _getDateDaysAgo(days: number){
    const toReturn = new Date();
    toReturn.setDate(toReturn.getDate() - days);

    // mi interessa solo il giorno
    return new Date(toReturn.toDateString());
  }
}

