import { Directive, HostListener, ElementRef, Input } from '@angular/core';
import { Subscription } from 'rxjs';
import { UserSearchService } from 'src/app/core/_services/user-search.service';
import { AutoCompleteComponent } from '@syncfusion/ej2-angular-dropdowns';
import { Query, Predicate } from '@syncfusion/ej2-data';

@Directive({
  selector: '[searchUsersAC]'
})
export class SearchUsersAutoCompleteDirective {

  @Input()
  includeDisabled:any;

  @Input()
  selectedUserLevels:any;

  private subscriptions = new Subscription();

  constructor(private usersSearchServ: UserSearchService, private el: ElementRef,
    private host:AutoCompleteComponent) {
      //host.filterType = 'Contains';
     }

  @HostListener('filtering', ['$event']) Filtering(evt) {

    evt.preventDefaultAction=true;

    this.subscriptions.add(this.usersSearchServ
      .search(evt.text, this._getSelectedUserLevels(), this._getIncludeDisabled())
      .subscribe(items => {
        if (!items){
          return;
        }

        const predicate = new Predicate(this.host.fields.text, 'contains', evt.text, true);
        const query = new Query().where(predicate);

        evt.updateData(items, query);
    }));
  }

  private _getIncludeDisabled(){
    if (this.includeDisabled || this.includeDisabled === ''){
      return true;
    }
    return false;
  }

  private _getSelectedUserLevels(){
    if (this.selectedUserLevels){
      return this.selectedUserLevels as number[];
    }

    return null;
  }

  ngOnDestroy() {
    if (this.subscriptions){
      // unsubscribe to ensure no memory leaks
      this.subscriptions.unsubscribe();
    }
  }
}
