import { Directive, ElementRef, HostListener, Input } from '@angular/core';
import { UserSearchService } from 'src/app/core/_services/user-search.service';
import { Subscription } from 'rxjs';
import { DropDownListComponent } from '@syncfusion/ej2-angular-dropdowns';

@Directive({
  selector: '[searchUsersDDL]'
})
export class SearchUsersDropDownListDirective {

  @Input()
  includeDisabled:any;

  @Input()
  selectedUserLevels:any;

  private subscriptions = new Subscription();

  constructor(private usersSearchServ: UserSearchService, private el: ElementRef,
    private host:DropDownListComponent) {
      host.allowFiltering = true;
      host.filterType = 'Contains';
     }

  @HostListener('filtering', ['$event']) Filtering(evt) {

    this.subscriptions.add(this.usersSearchServ
      .search(evt.text, this._getSelectedUserLevels(), this._getIncludeDisabled())
      .subscribe(items => {
        if (!items){
          return;
        }

        if (!this.host.value)
        {
          this.host.dataSource = items;
          return;
        }

        const dataSource =  this.host.dataSource as Array<any>;
        const selectedIds = [this.host.value];

        const previousSelectedItems = dataSource
        .filter(m => selectedIds.find(s => s === m.Id));

        const newItems = items
        .filter(m => !previousSelectedItems.find(s => s.Id === m.Id));

        this.host.dataSource = [...newItems, ...previousSelectedItems];
    }));
  }

  private _getIncludeDisabled(){
    if (this.includeDisabled || this.includeDisabled === ''){
      return true;
    }
    return false;
  }

  private _getSelectedUserLevels(){
    if (this.selectedUserLevels){
      return this.selectedUserLevels as number[];
    }

    return null;
  }

  ngOnDestroy() {
    if (this.subscriptions){
      // unsubscribe to ensure no memory leaks
      this.subscriptions.unsubscribe();
    }
  }
}
