import { Directive, HostListener, ElementRef, Input } from '@angular/core';
import { Subscription } from 'rxjs';
import { UserSearchService } from 'src/app/core/_services/user-search.service';
import { MultiSelectComponent } from '@syncfusion/ej2-angular-dropdowns';

@Directive({
  selector: '[searchUsersMS]'
})
export class SearchUsersMultiSelectDirective {

  @Input()
  includeDisabled:any;

  @Input()
  selectedUserLevels:any;

  private subscriptions = new Subscription();

  constructor(private usersSearchServ: UserSearchService, private el: ElementRef,
    private host:MultiSelectComponent) {
      host.filterType = 'Contains';
     }

  @HostListener('actionBegin', ['$event']) ActionBegin(evt){
    // faccio una copia, perchè il componente potrebbe resettarli se non li trova nel datasource
    const selectedValues = (this.host.value) 
    ? [...this.host.value as Array<number>] 
    : this.host.value as Array<number>;

    // se non ci sono valori preselezionati, non devo fare nulla
    if (selectedValues === undefined || !selectedValues || selectedValues.length <= 0){
      return;
    }

    const dataSource = this.host.dataSource as Array<any>;

    // se ho valori preselezionati, ma sono tutti presenti nel datasource,
    // non devo fare nulla
    if (!!dataSource && !selectedValues.find(m => !dataSource.find(s => s.Id === m))){
      return;
    }

    // se arrivo qui, vuol dire che ho valori selezionati che non sono presenti nel datasource
    this.subscriptions.add(this.usersSearchServ
      .search(null, this._getSelectedUserLevels(), this._getIncludeDisabled(), selectedValues)
      .subscribe(items => {
        if (!items){
          return;
        }

        // nel frattempo il componente li può avere resettati
        this.host.value = selectedValues;

        const currentValue = this.host.value as Array<number>;
        const currentDataSource = this.host.dataSource as Array<any>;
        const extractedDataSource = items as Array<any>;

        this.host.dataSource = this._getNewDatasource(currentValue, currentDataSource, extractedDataSource);
    }));
  }

  @HostListener('filtering', ['$event']) Filtering(evt) {

    this.subscriptions.add(this.usersSearchServ
      .search(evt.text, this._getSelectedUserLevels(), this._getIncludeDisabled())
      .subscribe(items => {
        if (!items){
          return;
        }

        const currentValue = this.host.value as Array<number>;
        const currentDataSource = this.host.dataSource as Array<any>;
        const extractedDataSource = items as Array<any>;

        this.host.dataSource = this._getNewDatasource(currentValue, currentDataSource, extractedDataSource);
    }));
  }

  private _getNewDatasource(currentValue: Array<number>, currentDataSource: Array<any>, extractedDataSource: Array<any>){
    if (!this.host.value)
    {
      return extractedDataSource;
    }

    const previousSelectedItems = currentDataSource
    .filter(m => currentValue.find(s => s === m.Id));

    const newItems = extractedDataSource
    .filter(m => !previousSelectedItems.find(s => s.Id === m.Id));

    return [...newItems, ...previousSelectedItems];
  }

  private _getIncludeDisabled(){
    if (this.includeDisabled || this.includeDisabled === ''){
      return true;
    }
    return false;
  }

  private _getSelectedUserLevels(){
    if (this.selectedUserLevels){
      return this.selectedUserLevels as number[];
    }

    return null;
  }

  ngOnDestroy() {
    if (this.subscriptions){
      // unsubscribe to ensure no memory leaks
      this.subscriptions.unsubscribe();
    }
  }
}
