import { NgModule, ModuleWithProviders } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { RouterModule } from '@angular/router';
import { TranslateModule } from '@ngx-translate/core';
import { GridModule } from '@syncfusion/ej2-angular-grids';
import { ButtonModule, SwitchModule, CheckBoxModule } from '@syncfusion/ej2-angular-buttons';
import { MultiSelectAllModule, DropDownListModule, AutoCompleteModule } from '@syncfusion/ej2-angular-dropdowns';
import { DateTimePickerModule, DatePickerModule, TimePickerModule, DateRangePickerModule } from '@syncfusion/ej2-angular-calendars';
import { NumericTextBoxModule, UploaderModule } from '@syncfusion/ej2-angular-inputs';
import {TextBoxModule} from '@syncfusion/ej2-angular-inputs';
import { DialogModule } from '@syncfusion/ej2-angular-popups';
import { MenuModule, TabModule } from '@syncfusion/ej2-angular-navigations';
import { FileManagerModule } from '@syncfusion/ej2-angular-filemanager';
import { GridsterModule } from 'angular-gridster2';
import { NfcReaderComponent } from './components/nfc-reader/nfc-reader.component';
import { CollapseModule } from 'ngx-bootstrap/collapse';
import { PanelExpanderComponent } from './components/panel-expander/panel-expander.component';
import { ArchwizardModule } from 'angular-archwizard';
import { CardGridComponent } from './components/card-grid/card-grid.component';
import { PopoverModule } from 'ngx-bootstrap/popover';
import { SearchProdottiDropDownListDirective } from './directives/search-prodotti-drop-down-list.directive';
import { SearchGruppiDropDownListDirective } from './directives/search-gruppi-drop-down-list.directive';
import { SearchParticolariDropDownListDirective } from './directives/search-particolari-drop-down-list.directive';
import { SearchComponentiDropDownListDirective } from './directives/search-componenti-drop-down-list.directive';
import { SearchAnomalieDropDownListDirective } from './directives/search-anomalie-drop-down-list.directive';
import { SearchPosizioniAnomalieDropDownListDirective } from './directives/search-posizioni-anomalie-drop-down-list.directive';
import { SearchPesiDifettiDropDownListDirective } from './directives/search-pesi-difetti-drop-down-list.directive';
import { SearchUsersDropDownListDirective } from './directives/search-users-drop-down-list.directive';
import { SearchUsersMultiSelectDirective } from './directives/search-users-multi-select.directive';
import { SearchUsersAutoCompleteDirective } from './directives/search-users-autocomplete.directive';
import { DynamicModule } from 'ng-dynamic-component';
import { DashboardComponent } from './components/dashboard/dashboard.component';
import { DashboardItemComponent } from './components/dashboard/dashboard-item/dashboard-item.component';
import { SmartRangepickerComponent } from './components/smart-rangepicker/smart-rangepicker.component';
import {WebcamModule} from 'ngx-webcam';
import { WebcamViewerComponent } from './components/webcam-viewer/webcam-viewer.component';
import { FileManagerComponent } from './components/file-manager/file-manager.component';
import { ContentHeadingComponent } from './components/content-heading/content-heading.component';
import { ReportViewerComponent } from './components/report-viewer/report-viewer.component';
import { NgxExtendedPdfViewerModule } from 'ngx-extended-pdf-viewer';
import { Am4chartsWrapperComponent } from './components/am4charts-wrapper/am4charts-wrapper.component';
import { DashboardItemSubtitleHeaderComponent } from './components/dashboard/dashboard-item/dashboard-item-subtitle-header/dashboard-item-subtitle-header.component';
import { DashboardItemFilterButtonComponent } from './components/dashboard/dashboard-item/dashboard-item-filter-button/dashboard-item-filter-button.component';
import { DashboardPanelsMenuComponent } from './components/dashboard/dashboard-panels-menu/dashboard-panels-menu.component';
import { NumericTextboxNotEmptyDirective } from './directives/numeric-textbox-not-empty.directive';
import { DragDropModule } from '@angular/cdk/drag-drop';
import { LeafletModule } from '@asymmetrik/ngx-leaflet';
import { WebcamStandaloneViewerComponent } from './components/webcam-standalone-viewer/webcam-standalone-viewer.component';
import { ImageDrawingModule } from 'ngx-image-drawing';
import { GridDifettiSessioneComponent } from './components/grid-difetti-sessione/grid-difetti-sessione.component';
import { HeaderSessioneComponent } from './components/header-sessione/header-sessione.component';
import { DebounceClickDirective } from './directives/debounce-click.directive';
import { MiniMenuComponent } from './components/mini-menu/mini-menu.component';
import { PwaStatusComponent } from './components/pwa-status/pwa-status.component';

// https://angular.io/styleguide#!#04-10
@NgModule({
    imports: [
        CommonModule,
        FormsModule,
        ReactiveFormsModule,
        DragDropModule,
        TranslateModule,
        ButtonModule,
        SwitchModule,
        DialogModule,
        TabModule,
        GridModule,
        DatePickerModule,
        DateTimePickerModule,
        NumericTextBoxModule,
        TextBoxModule,
        DropDownListModule,
        MultiSelectAllModule,
        GridsterModule,
        DynamicModule,
        DateRangePickerModule,
        FileManagerModule,
        MenuModule,
        NgxExtendedPdfViewerModule,
        WebcamModule,
        LeafletModule,
        CollapseModule.forRoot(),
        PopoverModule.forRoot(),
        ImageDrawingModule
    ],
    providers: [
    ],
    declarations: [
        ContentHeadingComponent,
        NfcReaderComponent,
        PanelExpanderComponent,
        CardGridComponent,
        DashboardComponent,
        DashboardItemComponent,
        DashboardItemSubtitleHeaderComponent,
        DashboardItemFilterButtonComponent,
        DashboardPanelsMenuComponent,
        SmartRangepickerComponent,
        FileManagerComponent,
        WebcamStandaloneViewerComponent,
        WebcamViewerComponent,
        ReportViewerComponent,
        Am4chartsWrapperComponent,
        HeaderSessioneComponent,
        GridDifettiSessioneComponent,
        MiniMenuComponent,
        PwaStatusComponent,
        SearchProdottiDropDownListDirective,
        SearchGruppiDropDownListDirective,
        SearchParticolariDropDownListDirective,
        SearchComponentiDropDownListDirective,
        SearchAnomalieDropDownListDirective,
        SearchPosizioniAnomalieDropDownListDirective,
        SearchPesiDifettiDropDownListDirective,
        SearchUsersDropDownListDirective,
        SearchUsersMultiSelectDirective,
        SearchUsersAutoCompleteDirective,
        NumericTextboxNotEmptyDirective,
        DebounceClickDirective,
    ],
    exports: [
        CommonModule,
        FormsModule,
        ReactiveFormsModule,
        DragDropModule,
        TranslateModule,
        RouterModule,
        DynamicModule,
        // EJ2 Syncfusion
        GridModule,
        ButtonModule,
        SwitchModule,
        // MultiSelectModule,
        // il seguente modulo serve
        // per far funzionare correttamente
        // i componenti multiselect di EJ2
        // in modalità CheckBox
        DropDownListModule,
        MultiSelectAllModule,
        AutoCompleteModule,
        DatePickerModule,
        DateTimePickerModule,
        DatePickerModule,
        TimePickerModule,
        DateRangePickerModule,
        NumericTextBoxModule,
        TextBoxModule,
        SwitchModule,
        CheckBoxModule,
        DialogModule,
        TabModule,
        UploaderModule,
        FileManagerModule,
        MenuModule,
        NgxExtendedPdfViewerModule,
        GridsterModule,
        CollapseModule,
        ArchwizardModule,
        PopoverModule,
        WebcamModule,
        LeafletModule,
        // COMPONENTI COMUNI
        // DEFINITI IN SHARED
        ContentHeadingComponent,
        DashboardComponent,
        NfcReaderComponent,
        PanelExpanderComponent,
        CardGridComponent,
        SmartRangepickerComponent,
        FileManagerComponent,
        WebcamStandaloneViewerComponent,
        WebcamViewerComponent,
        ReportViewerComponent,
        Am4chartsWrapperComponent,
        HeaderSessioneComponent,
        GridDifettiSessioneComponent,
        MiniMenuComponent,
        PwaStatusComponent,
        SearchProdottiDropDownListDirective,
        SearchGruppiDropDownListDirective,
        SearchParticolariDropDownListDirective,
        SearchComponentiDropDownListDirective,
        SearchAnomalieDropDownListDirective,
        SearchPosizioniAnomalieDropDownListDirective,
        SearchPesiDifettiDropDownListDirective,
        SearchUsersDropDownListDirective,
        SearchUsersMultiSelectDirective,
        SearchUsersAutoCompleteDirective,
        NumericTextboxNotEmptyDirective,
        DebounceClickDirective
    ]
})

// https://github.com/ocombe/ng2-translate/issues/209
export class SharedModule {
    static forRoot(): ModuleWithProviders<SharedModule> {
        return {
            ngModule: SharedModule,
            providers: [],
        };
    }
}
